/*
 * Google Maps JavaScript API
 *
 * CLASSI UTILIZZATE
 *
 *   google.maps.InfoWindow   : Finestra informativa per localita' geografica.
 *   google.maps.LatLng       : Localita' geografica con coordinate (latitudine-nord, longitudine-est).
 *   google.maps.LatLngBounds : Rettangolo geografico individuato dai vertici (sud-ovest, nord-est).
 *   google.maps.Map          : Mappa geografica.
 *   google.maps.Marker       : Marcatore di una localita' geografica.
 *   google.maps.Point        : Punto di un'immagine.
 *   google.maps.Size         : Dimensione di un'immagine (larghezza, altezza)
 *
 * OGGETTI UTILIZZATI (non hanno il costruttore)
 *
 *   google.maps.MapOptions    : Opzioni mappa.
 *   google.maps.MarkerOptions : Opzioni marcatore.
 *   google.maps.Icon          : Opzioni icona di marcatore.
 */



////////////////////////////////////////////////////////////////////////////////
//
//   G e o L o c a t i o n
//
////////////////////////////////////////////////////////////////////////////////


/**
 * Oggetto GeoLocation : Localita' geografica.
 *
 * @param float lat Latitudine nord.
 * @param float lng Longitudine est.
 * @param string name Denominazione della localita' geografica.
 * @param string info Informazioni sulla localita' geografica.
 * @param string id Identificativo univoco della localita' geografica.
 * @param string label Etichetta di una singola lettera per il marcatore.
 */
function GeoLocation(lat, lng, name, info, id, label) {
  this.latlng = new google.maps.LatLng(lat, lng);
  this.name = (typeof name !== 'undefined' ? name : null);
  this.info = (typeof info !== 'undefined' ? info : null);
  this.id = (typeof id !== 'undefined' ? id : null);
  this.label = (typeof label !== 'undefined' ? label : null);
  this.marker = null;
}


/**
 * Restituisce la latitudine nord della localita' geografica.
 *
 * @return float Latitudine nord della localita'.
 */
GeoLocation.prototype.lat = function() {
  return this.latlng.lat();
}


/**
 * Restituisce la longitudine est della localita' geografica.
 *
 * @return float Longitudine est della localita'.
 */
GeoLocation.prototype.lng = function() {
  return this.latlng.lng();
}


/**
 * Restituisce l'oggetto LatLng della localita' geografica.
 *
 * @return LatLng Oggetto LatLng della localita'.
 */
GeoLocation.prototype.getLatLng = function() {
  return this.latlng;
}


/**
 * Imposta il marcatore di localita' geografica aggiunto alla mappa.
 *
 * @param google.maps.Marker marker Marcatore della localita' geografica.
 */
GeoLocation.prototype.setMarker = function(marker) {
  this.marker = marker;
}


////////////////////////////////////////////////////////////////////////////////
//
//   G e o R e c t a n g l e
//
////////////////////////////////////////////////////////////////////////////////


/**
 * Oggetto GeoRectangle : Rettangolo geografico.
 *
 * @param GeoLocation sw Coordinate del punto sud-ovest.
 * @oaram GeoLocation ne Coordinate del punto nord-est.
 */
function GeoRectangle(sw, ne) {
  this.latlngbounds = new google.maps.LatLngBounds(sw.getLatLng(), ne.getLatLng());
}


/**
 * Restituisce l'altezza del rettangolo geografico come angolo di longitudine.
 *
 * @return float Altezza in gradi.
 */
GeoRectangle.prototype.getHeight = function() {
  return Math.abs(this.latlngbounds.getNorthEast().lng() - this.latlngbounds.getSouthWest().lng());
}


/**
 * Restituisce la larghezza del rettangolo geografico come angolo di latitudine.
 *
 * @return float Larghezza in gradi.
 */
GeoRectangle.prototype.getWidth = function() {
  return Math.abs(this.latlngbounds.getNorthEast().lat() - this.latlngbounds.getSouthWest().lat());
}


/**
 * Restituisce l'oggetto LatLngBounds del rettangolo geografico.
 *
 * @return LatLngBounds Oggetto LatLngBounds del rettangolo.
 */
GeoRectangle.prototype.getLatLngBounds = function() {
  return this.latlngbounds;
}


////////////////////////////////////////////////////////////////////////////////
//
//   G e o M a p
//
////////////////////////////////////////////////////////////////////////////////


var GEO_DEFAULT_LOCATION = {lat:41.909986, lng:12.3959154, name:'Roma'};

var GEO_ZOOM_WORLD = 1;
var GEO_ZOOM_LANDMASS_CONTINENT = 5;
var GEO_ZOOM_CITY = 10;
var GEO_ZOOM_STREETS = 15;
var GEO_ZOOM_BUILDINGS = 20;


/**
 * Oggetto GeoMap : Mappa geografica.
 *
 * @param HTMLDivElement container DIV destinato a contenere la mappa geografica.
 * @param GeoLocation center Centro della mappa geografica.
 * @param integer zoom Livello di zoom.
 * @param GeoLocation[] locations Elenco delle localita'.
 */
function GeoMap(container, center, zoom, locations) {
  this.container = container;

  if (typeof center == 'undefined')
    this.center = new GeoLocation(GEO_DEFAULT_LOCATION.lat, GEO_DEFAULT_LOCATION.lng, GEO_DEFAULT_LOCATION.name);
  else
    this.center = center;

  this.zoom = (typeof zoom !== 'undefined' ? zoom : GEO_ZOOM_STREETS);
  this.locations = (typeof locations !== 'undefined' ? locations : [GEO_DEFAULT_LOCATION]);
  this.targetLocation = null;
  this.map = null;
  this.markers = null;
  this.icon = null;
  this.targetIcon = null;
  this.infoWinAnchor = null;
  this.locationsArea = null;
  this.areaView = false;
}


/**
 * Aggiunge un marcatore di localita' alla mappa.
 *
 * @param GeoLocation location Localita' da marcare.
 * @param google.maps.Icon icon Icona del marcatore.

 * @param boolean save Flag salva il marcatore nella localita'.
 */
GeoMap.prototype.addMarker = function(location, icon, save) {
  var options = new google.maps.MarkerOptions;

  if (typeof anchor_point !== 'undefined')
    options.anchorPoint = anchor_point;
  else if (this.anchorPoint !== null)
    options.anchorPoint = this.anchorPoint;

  if (typeof icon !== 'undefined')
    options.icon = icon;
  else if (this.icon !== null)
    options.icon = this.icon;

  if (typeof label !== 'undefined')
    options.label = label;
  else if (this.label !== null)
    options.label = this.label;

  options.map = this.map;

  if (typeof location.name !== 'undefined')
    options.title = location.name;

  options.position = location;

  var marker = new google.maps.Marker(options);

  if (typeof save == 'boolean' && save)
    location.setMarker(marker)
}


/**
 * Crea la mappa all'interno del contenitore ad essa assegnato.
 */
GeoMap.prototype.init = function() {
  this.map = new google.maps.Map(this.container, {
    center: this.center.getLatLng(),
    zoom: this.zoom
  });

  if (Array.isArray(this.locations)) {
    this.markers = [];

    for (var i = 0; i < this.locations.length; i++) {
      this.markers[i] = new google.maps.Marker({
        anchorPoint: this.infoWinAnchor,
        icon: this.icon,
        map: this.map,
        title: this.locations[i].name,
        position: this.locations[i].getLatLng()
        //TODO label : si potrebbe inserire nella GeoLocation 'A'
      });

      if (this.locations[i].info != null) {
        this.markers[i].addListener('click', (function(_map, _marker, _content) {
          var info_window = new google.maps.InfoWindow({
            position: _marker.position,
            content: _content
          });
          return function() {
            info_window.open(_map, _marker);
          };
        })(this.map, this.markers[i], this.locations[i].info));
      }
    }

    if (this.locations.length > 1) {
      // Creo il rettangolo minimo che contiene tutte le localita' della mappa
      this.locationsArea = GeoMap.locationsRectangle(this.locations);
      this.zoomArea();
    }
  }
}


/**
 * Restituisce il centro geografico di un elenco di localita'.
 *
 * @param GeoLocation[] locations Elenco delle localita'.
 * @param string name Denominazione del centro geograficodelle localita'.
 *
 * @return mixed false=Errore, GeoLocation: centro geografico delle localita'.
 */
GeoMap.locationsCenter = function(locations, name) {

  if (!Array.isArray(locations))
    return false;

  if (locations.length == 1)
    return locations[0];

  var rectangle = GeoMap.locationsRectangle(locations);

  if (rectangle === false)
    return false;

  var ne = rectangle.getLatLngBounds().getNorthEast();
  var sw = rectangle.getLatLngBounds().getSouthWest();

  return new GeoLocation(
    (rectangle.getWidth() > 0.000001 ? (ne.lat() + sw.lat()) / 2 : ne.lat()),
    (rectangle.getHeight() > 0.000001 ? (ne.lng() + sw.lng()) / 2 : ne.lng()),
    name
  );
}


/**
 * Restituisce il rettangolo che contiene un elenco di localita'.
 *
 * @param GeoLocation[] locations Elenco delle localita'.
 *
 * @return mixed false=Errore, GeoRectangle: rettangolo delle localita''.
 */
GeoMap.locationsRectangle = function(locations) {

  if (!Array.isArray(locations) || locations.length < 2)
    return false;

  var lat_max = -90, lat_min = 90, lng_max = -180, lng_min = 180, cur_lat, cur_lng;

  for (var i = 0; i < locations.length; i++) {
    cur_lat = locations[i].lat();
    cur_lng = locations[i].lng();

    if (cur_lat > lat_max)
      lat_max = cur_lat;

    if (cur_lat < lat_min)
      lat_min = cur_lat;

    if (cur_lng > lng_max)
      lng_max = cur_lng;

    if (cur_lng < lng_min)
      lng_min = cur_lng;
  }

  return new GeoRectangle(new GeoLocation(lat_min, lng_min),
                          new GeoLocation(lat_max, lng_max));
}


/**
 * Crea un'icona per un marcatore.
 *
 * @param string url URL dell'icona.
 * @param integer width Larghezza dell'icona.
 * @param integer height Altezza dell'icona.
 * @param integer anchor_x Ascissa del punto di ancoraggio dell'icona al marcatore riferita al margine sinistro dell'icona.
 * @param integer anchor_y Ordinata del punto di ancoraggio dell'icona al marcatore riferita al margine superiore dell'icona.
 * @param integer label_x Ascissa del punto su cui centrare l'etichetta del marcatore riferita al margine sinistro dell'icona.
 * @param integer label_y Ordianta del punto su cui centrare l'etichetta del marcatore riferita al margine superiore dell'icona.
 *
 * @return google.maps.Icon Icona per il marcatore.
 */
GeoMap.prototype.markerIcon = function(url, width, height, anchor_x, anchor_y, label_x, label_y) {
  var icon = new Object; //google.maps.Icon

  if (typeof(anchor_x) !== 'undefined' && typeof(anchor_y) != 'undefined')
    icon.anchor = new google.maps.Point(anchor_x, anchor_y);

  if (typeof(label_x) !== 'undefined' && typeof(label_y) != 'undefined')
    icon.labelOrigin = new google.maps.Point(label_x, label_y)

  if (typeof(width) !== 'undefined' && typeof(height) != 'undefined') {
    icon.scaledSize = new google.maps.Size(width, height);
    icon.size = new google.maps.Size(width, height);
  }

  icon.url = url;
  return icon;
}


/**
 * Imposta il punto di ancoraggio della finestra informativa.
 *
 * @param integer x Ascissa relativa al punto di ancoraggio dell'icona.
 * @param integer y Ordinata relativa al punto di ancoraggio dell'icona.
 */
GeoMap.prototype.setInfoWinAnchor = function(x, y) {
  this.infoWinAnchor = new google.maps.Point(x, y);
}


/**
 * Imposta l'icona dei marcatori.
 *
 * @param string url URL dell'icona.
 * @param integer width Larghezza dell'icona.
 * @param integer height Altezza dell'icona.
 * @param integer anchor_x Ascissa del punto di ancoraggio dell'icona al marcatore riferita al margine sinistro dell'icona.
 * @param integer anchor_y Ordinata del punto di ancoraggio dell'icona al marcatore riferita al margine superiore dell'icona.
 * @param integer label_x Ascissa del punto su cui centrare l'etichetta del marcatore riferita al margine sinistro dell'icona.
 * @param integer label_y Ordianta del punto su cui centrare l'etichetta del marcatore riferita al margine superiore dell'icona.
 */
GeoMap.prototype.setIcon = function(url, width, height, anchor_x, anchor_y, label_x, label_y) {
  this.icon = this.markerIcon(url, width, height, anchor_x, anchor_y, label_x, label_y);
}


/**
 * Imposta l'icona del marcatore della localita' sulla quale viene centrata la mappa.
 *
 * @param string url URL dell'icona.
 * @param integer width Larghezza dell'icona.
 * @param integer height Altezza dell'icona.
 * @param integer anchor_x Ascissa del punto di ancoraggio dell'icona al marcatore riferita al margine sinistro dell'icona.
 * @param integer anchor_y Ordinata del punto di ancoraggio dell'icona al marcatore riferita al margine superiore dell'icona.
 * @param integer label_x Ascissa del punto su cui centrare l'etichetta del marcatore riferita al margine sinistro dell'icona.
 * @param integer label_y Ordianta del punto su cui centrare l'etichetta del marcatore riferita al margine superiore dell'icona.
 */
GeoMap.prototype.setTargetIcon = function(url, width, height, anchor_x, anchor_y, label_x, label_y) {
  this.targetIcon = this.markerIcon(url, width, height, anchor_x, anchor_y, label_x, label_y);
}


/**
 * Effettua lo zoom sul rettangolo minimo che contiene tutte le localita' della mappa.
 */
GeoMap.prototype.zoomArea = function() {
  if (typeof this.locationsArea == null)
    return;

  this.map.setCenter(this.center.getLatLng());
  this.map.setZoom(GEO_ZOOM_BUILDINGS);
  this.map.fitBounds(this.locationsArea.getLatLngBounds());
  this.areaView = true;

  if (this.targetIcon !== null && this.targetLocation !== null) {
    this.markers[this.targetLocation].setIcon(this.icon);
    this.targetLocation = null;
  }
}


/**
 * Effettua lo zoom su una delle localita' della mappa.
 *
 * @param string id Identificativo univoco della localita' geografica.
 */
GeoMap.prototype.zoomLocation = function(id) {
  for (var i = 0; i < this.locations.length; i++)
    if (this.locations[i].id == id)
      break;

  if (i == this.locations.length)
    return;

  this.map.setCenter(this.locations[i].getLatLng());
  this.map.setZoom(GEO_ZOOM_STREETS);
  this.areaView = false;

  if (this.targetIcon !== null) {
    if (this.targetLocation !== null)
      this.markers[this.targetLocation].setIcon(this.icon);

    this.targetLocation = i;
    this.markers[this.targetLocation].setIcon(this.targetIcon);
  }
}


/**
 * Effettua lo zoom sul rettangolo minimo che contiene tutte le localita' della mappa.
 */
GeoMap.prototype.zoomRect = function(list) {
  locationsArea = GeoMap.locationsRectangle(list);
  center = GeoMap.locationsCenter(list);

  this.map.setCenter(center.getLatLng());
  this.map.setZoom(GEO_ZOOM_CITY);
  this.map.fitBounds(locationsArea.getLatLngBounds());
  this.areaView = true;

}

